from .map import Map
from .layer import LayerGroup, FeatureGroup, imageOverlay
from .layer.tile import TileLayer
from .layer.marker import Marker
from .layer.icon import Icon
from .layer.vector import Circle, CircleMarker, Polygon, Polyline, Rectangle
from .control import Control

class L:
    """
    Leaflet namespace that holds reference to all the leaflet objects
    """
    map = Map
    tileLayer = TileLayer
    imageOverlay = imageOverlay
    marker = Marker
    icon = Icon
    circleMarker = CircleMarker
    polyline = Polyline
    polygon = Polygon
    rectangle = Rectangle
    circle = Circle
    layerGroup = LayerGroup
    featureGroup = FeatureGroup
    control = Control




from ..layer import Layer
from ..icon import Icon
from ...core.Parser import Parser

from qtpy.QtCore import Slot, Signal, QJsonValue

from typing import List


class Marker(Layer):
    moveend = Signal(dict)
    move = Signal(dict)
    click = Signal(dict)

    def __init__(self, latLng: List[float], options=None):
        super().__init__()
        if isinstance(options, type(None)):
            options = {}
        self.latLng = latLng
        self.options = options
        self.opacity = options.get('opacity', 1)
        self.draggable = options.get('draggable', False)
        if self._map:
            self._initJs()

    @Slot(QJsonValue)
    def _onMove(self, event):
        self._logger.debug('marker moved. event: {event}'.format(event=event))
        event = self._qJsonValueToDict(event)
        self.latLng = [event["latlng"]["lat"], event["latlng"]["lng"]]
        self.move.emit({**self._qJsonValueToDict(event), "latLng": self.latLng, "sender": self})

    @Slot(QJsonValue)
    def _onMoveend(self, event):
        self._logger.debug('marker moved. event: {event}'.format(event=event))
        if self.opacity == 0:
            return
        self.moveend.emit({**self._qJsonValueToDict(event), "latLng": self.latLng, "sender": self})

    @Slot(QJsonValue)
    def _click(self, event):
        self._logger.debug('marker clicked. event: {event}'.format(event=event))
        if self.opacity == 0:
            return
        self.click.emit({**self._qJsonValueToDict(event), "sender": self})

    def _initJs(self):
        leafletJsObject = 'L.marker({latLng}'.format(latLng=self.latLng)
        if self.options:
            leafletJsObject += ', {options}'.format(options=Parser.dict_for_js(self.options))
        leafletJsObject += ')'
        self._createJsObject(leafletJsObject, self._map.mapWidgetIndex)
        self._connectEventToSignal('move', '_onMove', self._map.mapWidgetIndex)
        self._connectEventToSignal('moveend', '_onMoveend', self._map.mapWidgetIndex)
        self._connectEventToSignal('click', '_click', self._map.mapWidgetIndex)

    def setLatLng(self, latLng):
        self.latLng = latLng
        js = '{layerName}.setLatLng({latLng})'.format(
                layerName=self._layerName, latLng=latLng)
        self.runJavaScriptForMapIndex(js)
        return self

    def setOpacity(self, opacity):
        self.opacity = opacity
        js = '{layerName}.setOpacity({opacity})'.format(
                layerName=self._layerName, opacity=self.opacity)
        self.runJavaScriptForMapIndex(js)
        return self

    def setDragging(self, draggable):
        self.draggable = draggable
        option = 'enable' if self.draggable else 'disable'
        js = '{layerName}.dragging.{option}();'.format(layerName=self._layerName, option=option)
        self.runJavaScriptForMapIndex(js)
        return self

    def setIcon(self, icon: Icon):
        js = '{layerName}.setIcon({markerIcon});'.format(layerName=self._layerName, markerIcon=icon._layerName)
        self.runJavaScriptForMapIndex(js)
        return self

    def setRotationAngle(self, angle_deg: float):
        js = '{layerName}.setRotationAngle({angle_deg});'.format(layerName=self._layerName, angle_deg=angle_deg)
        self.runJavaScriptForMapIndex(js)
        return self

    def setRotationOrigin(self, origin: str):
        js = '{layerName}.setRotationOrigin({origin});'.format(layerName=self._layerName, origin=origin)
        self.runJavaScriptForMapIndex(js)
        return self



from ..core import Evented
import logging
from abc import abstractmethod

class Layer(Evented):

    # layerId is a static variable shared between all layers
    # It is used to give unique names to layers
    layerId = 0

    @property
    def layerName(self):
        return self._layerName

    @layerName.setter
    def layerName(self, name):
        self._layerName = name

    @property
    def jsName(self):
        return self._layerName

    @property
    def map(self):
        return self._map

    @map.setter
    def map(self, map_):
        self._map = map_

    @abstractmethod
    def _initJs(self):
        raise NotImplemented

    def runJavaScriptForMapIndex(self, js):
        if self._map is not None:
            self.runJavaScript(js, self._map.mapWidgetIndex)

    def __init__(self):
        super().__init__()
        self._map = None
        self._layerName = self._getNewLayerName()
        self._log = logging.getLogger(f"layer_{self._layerName}")
        self._popup = None
        self._popupOptions = None
        self._tooltip = None
        self._tooltipOptions = None

    def _initPopupAndTooltip(self):
        if self._popup is not None:
            self._bindPopupOrTooltip("Popup", self._popup, self._popupOptions)
        if self._tooltip is not None:
            self._bindPopupOrTooltip("Tooltip", self._tooltip, self._tooltipOptions)

    def _bindPopupOrTooltip(self, kind, content, options):
        js = f'{self._layerName}.bind{kind}("{content}"'
        if options is not None:
            js += f', {self._stringifyForJs(options)}'
        js += ')'
        self.runJavaScriptForMapIndex(js)

    def _getNewLayerName(self):
        layerName = 'l{}'.format(self.layerId)
        Layer.layerId += 1
        return layerName

    def addTo(self, map_):
        map_.addLayer(self)
        return self

    def removeFrom(self, map_):
        map_.removeLayer(self)
        return self

    def bindPopup(self, content, options=None):
        self._popup = content
        self._popupOptions = options
        self._bindPopupOrTooltip("Popup", self._popup, self._popupOptions)
        return self

    def unbindPopup(self):
        self._popup = None
        self._popupOptions = None
        js = '{layerName}.unbindPopup()'.format(layerName=self._layerName)
        self.runJavaScriptForMapIndex(js)
        return self

    def bindTooltip(self, content, options=None):
        self._tooltip = content
        self._tooltipOptions = options
        self._bindPopupOrTooltip("Tooltip", self._tooltip, self._tooltipOptions)
        return self

    def unbindTooltip(self):
        self._tooltip = None
        self._tooltipOptions = None
        js = '{layerName}.unbindTooltip()'.format(layerName=self._layerName)
        self.runJavaScriptForMapIndex(js)
        return self





from ..layer import Layer
from ...core.Parser import Parser
import os


class Icon(Layer):
    def __init__(self, iconUrl: str, options=None):
        super().__init__()
        if isinstance(options, type(None)):
            options = {}
        self.iconUrl = iconUrl
        self.icon_found = False
        self.options = options
        self._check_icon_url()
        if self._map:
            self._initJs()

    def _check_icon_url(self):
        if "http" in self.iconUrl:
            self._log.info("Can't check if icon exists at url!")
            return
        if not os.path.isfile(self.iconUrl):
            self._log.error(f"Can't locate file at path: '{self.iconUrl}'. Current working directory is '{os.getcwd()}'")
            return
        self.icon_found = True

    def _initJs(self):
        leafletJsObject = 'L.icon({options});'.format(options=Parser.dict_for_js({"iconUrl": self.iconUrl,
                                                                                  **self.options}))
        self._createJsObject(leafletJsObject, self._map.mapWidgetIndex)


from .circlemarker import CircleMarker


class Circle(CircleMarker):
    def __init__(self, latLng, radius, options=None):
        self.radius = radius
        super().__init__(latLng, options)

    def _initJs(self):
        leafletJsObject = 'L.circle({latLng},{radius}'.format(latLng=self.latLng,
                radius=self.radius)
        if self.options:
            leafletJsObject += ', {options}'.format(options=self.options)
        leafletJsObject += ')'
        self._createJsObject(leafletJsObject, self)



from .path import Path


class CircleMarker(Path):
    def __init__(self, latLng, options=None):
        super().__init__()
        self.latLng = latLng
        self.options = options
        if self._map:
            self._initJs()

    def _initJs(self):
        leafletJsObject = 'L.circleMarker({latLng}'.format(latLng=self.latLng)
        if self.options:
            leafletJsObject += ', {options}'.format(options=self.options)
        leafletJsObject += ')'
        self._createJsObject(leafletJsObject, self._map.mapWidgetIndex)




import json
import logging
import os
import time

from qtpy.QtCore import Slot, Signal, QJsonValue

from ... import mapwidget
from ..core import Evented
from ..layer import Layer


class Map(Evented):
    '''
    .. module:: pyqtlet2

    pyqtlet2 equivalent of L.map

    Map element has to be the first pyqtlet2 object to be initiated.

    .. note::
        Further documentation can be found at the official leaflet API.

    :param pyqtlet2.MapWidget mapWidget: The mapwidget object
        Should only be sent once, when the first object is being 
        initialised.

    :param dict options: Options for initiation (optional)
    '''

    clicked = Signal(dict)
    zoom = Signal(dict)
    drawCreated = Signal(dict)
    right_mouse_clicked = Signal(dict)
    mapWidgetCounter = 0

    @property
    def layers(self):
        """
        Instead of L.map.eachLayer
        """
        return self._layers

    @property
    def jsName(self):
        '''
        Name of the Leaflet element
        '''
        return self._jsName

    @Slot(QJsonValue)
    def _onClick(self, event):
        self._logger.debug('map clicked. event: {event}'.format(event=event))
        self.clicked.emit(self._qJsonValueToDict(event))
        
    @Slot(QJsonValue)
    def _onRightClick(self, event):
        self._logger.debug('map clicked. event: {event}'.format(event=event))
        self.right_mouse_clicked.emit(self._qJsonValueToDict(event))

    @Slot(QJsonValue)
    def _onDrawCreated(self, event):
        self._logger.debug('draw created. event: {event}'.format(event=event))
        self.drawCreated.emit(self._qJsonValueToDict(event))

    @Slot(QJsonValue)
    def _onZoom(self, event):
        self._logger.debug('map zoom. event: {event}'.format(event=event))
        self.zoom.emit(self._qJsonValueToDict(event))

    def __init__(self, mapWidget, options=None):
        '''
        pyqtlet2 equivalent of L.map

        Map element has to be the first pyqtlet2 object to be initiated.

        :param pyqtlet2.MapWidget mapWidget: The mapwidget object
            Should only be sent once, when the first object is being
            initialised.

        :param dict options: Options for initiation (optional)

        .. note
            Further documentation can be found at the official leaflet API.
        '''

        super().__init__(mapWidget, Map.mapWidgetCounter)
        self.mapWidgetIndex = Map.mapWidgetCounter
        Map.mapWidgetCounter += 1
        self._logger = logging.getLogger(__name__)
        self.options = options
        self._layers = []
        self._controls = []
        self._jsName = 'map'
        self._initJs()
        self._connectEventToSignal('click', '_onClick', self.mapWidgetIndex)
        self._connectEventToSignal('contextmenu', '_onRightClick', self.mapWidgetIndex)
        self._connectEventToSignal('zoom', '_onZoom', self.mapWidgetIndex)
        self._connectEventToSignal('draw:created', '_onDrawCreated', self.mapWidgetIndex)

    def _initJs(self):
        jsObject = 'L.map("map"'
        if self.options:
            jsObject += ', {options}'.format(options=self._stringifyForJs(self.options))
        jsObject += ')'
        self._createJsObject(jsObject, self.mapWidgetIndex)

    def setView(self, latLng, zoom=None, options=None):
        js = 'map.setView({latLng}'.format(latLng=latLng)
        if zoom:
            js += ', {zoom}'.format(zoom=zoom)
        if options:
            js += ', {options}'.format(options=options)
        js += ');'
        self.runJavaScript(js, self.mapWidgetIndex)
        return self

    def runJavaScriptForMap(self, js):
        self.runJavaScript(js, self.mapWidgetIndex)

    def addLayer(self, layer: Layer):
        self._layers.append(layer)
        layer.map = self
        layer._initJs()
        js = 'map.addLayer({layerName})'.format(layerName=layer.layerName)
        self.runJavaScriptForMap(js)
        layer._initPopupAndTooltip()
        return self

    def removeLayer(self, layer):
        if layer not in self._layers:
            # TODO Should we raise a ValueError here? Or just return
            return
        self._layers.remove(layer)
        layer.map = None
        js = 'map.removeLayer({layerName})'.format(layerName=layer.layerName)
        self.runJavaScriptForMap(js)
        return self

    def addControl(self, control):
        self._controls.append(control)
        control.map = self
        control._initJs()
        js = 'map.addControl({controlName})'.format(controlName=control.controlName)
        self.runJavaScriptForMap(js)
        return self

    def removeControl(self, control):
        if control not in self._controls:
            # TODO Should we raise a ValueError here? Or just return
            return
        self._controls.remove(control)
        control.map = None
        js = 'map.removeControl({controlName})'.format(controlName=control.controlName)
        self.runJavaScriptForMap(js)
        return self

    def getJsresponseForMap(self, js, callback):
        return self.getJsResponse(js, self.mapWidgetIndex, callback)

    def getBounds(self, callback):
        return self.getJsresponseForMap('map.getBounds()', callback)

    def getCenter(self, callback):
        return self.getJsresponseForMap('map.getCenter()', callback)

    def getZoom(self, callback):
        return self.getJsresponseForMap('map.getZoom()', callback)

    def getState(self, callback):
        return self.getJsresponseForMap('getMapState()', callback)

    def hasLayer(self, layer):
        return layer in self._layers

    def setZoom(self, zoom, options=None):
        js = 'map.setZoom({zoom}'.format(zoom=zoom)
        if options:
            js += ', {options}'.format(options=options)
        js += ');'
        self.runJavaScriptForMap(js)
        return self

    def setMaxBounds(self, bounds):
        js = 'map.setMaxBounds({bounds})'.format(bounds=bounds)
        self.runJavaScriptForMap(js)
        return self

    def fitBounds(self, bounds):
        js = 'map.fitBounds({bounds})'.format(bounds=bounds)
        self.runJavaScriptForMap(js)
        return self

    def setMaxZoom(self, zoom):
        js = 'map.setMaxZoom({zoom})'.format(zoom=zoom)
        self.runJavaScriptForMap(js)
        return self

    def setMinZoom(self, zoom):
        js = 'map.setMinZoom({zoom})'.format(zoom=zoom)
        self.runJavaScriptForMap(js)
        return self

    def panTo(self, latLng, options=None):
        js = 'map.panTo({latLng}'.format(latLng=latLng)
        if options:
            js += ', {options}'.format(options=options)
        js += ');'
        self.runJavaScriptForMap(js)
        return self

    def flyTo(self, latLng, zoom=None, options=None):
        js = 'map.flyTo({latLng}'.format(latLng=latLng)
        if zoom:
            js += ', {zoom}'.format(zoom=zoom)
        if options:
            js += ', {options}'.format(options=options)
        js += ');'
        self.runJavaScriptForMap(js)
        return self



import json
import logging
import os
import time

from qtpy.QtCore import Slot, Signal
from ..core import Evented

class Control(Evented):

    # controlId is a static variable shared between all controls
    # It is used to give unique names to controls
    controlId = 0
    # addedToMap and removedFromMap are signals for controls to
    # know when they're added and removed from maps
    addedToMap = Signal()
    removedFromMap = Signal()

    @property
    def map(self):
        return self._map

    @map.setter
    def map(self, map_):
        self._map = map_
        if map_ is None:
            self.removedFromMap.emit()
        else:
            self.addedToMap.emit()

    @property
    def jsName(self):
        return self._controlName

    @property
    def controlName(self):
        return self._controlName

    @controlName.setter
    def controlName(self, name):
        self._controlName = name

    def __init__(self):
        super().__init__()
        self._map = None
        self._controlName = self._getNewControlName()

    def addTo(self, map_):
        map_.addControl(self)
        return self

    def removeFrom(self, map_):
        map_.removeControl(self)

    def _getNewControlName(self):
        controlName = 'c{}'.format(self.controlId)
        Control.controlId += 1
        return controlName




from setuptools import setup, find_packages

with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name='pyqtlet2',
    version='0.9.2',
    description='Bringing leaflet maps to Python Qt bindings',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Leon Friedmann',
    author_email='leon.friedmann@tum.de',
    url='https://github.com/JaWeilBaum/pyqtlet2',
    keywords='leaflet, qtpy, maps, python, python3',
    classifiers=[],
    packages=[
        'pyqtlet2',
        'pyqtlet2.leaflet',
        'pyqtlet2.leaflet.control',
        'pyqtlet2.leaflet.core',
        'pyqtlet2.leaflet.layer',
        'pyqtlet2.leaflet.layer.marker',
        'pyqtlet2.leaflet.layer.tile',
        'pyqtlet2.leaflet.layer.vector',
        'pyqtlet2.leaflet.layer.icon',
        'pyqtlet2.leaflet.map',
    ],
    package_data={
        'pyqtlet2': [
            'web/map.html',
            'web/custom.js',
            'web/modules/*/*',
            'web/modules/*/images/*',
        ],
    },
    install_requires=[
        'QtPy>=2.0.1'
    ],
    extras_require = {
        'PyQt5' : ['PyQt5>=5.15.5','PyQtWebEngine>=5.15.5'],
        'PyQt6' : ['PyQt6>=6.2.0','PyQt6-WebEngine>=6.2.0'],
        'PySide2': ['PySide2'],
        'PySide6': ['PySide6']
    }
)





class Parser:
    @staticmethod
    def dict_for_js(object_to_parse: dict) -> dict:
        return_dict = {}
        for key, value in object_to_parse.items():
            if isinstance(value, bool):
                return_dict[key] = str(value).lower()
                continue

            return_dict[key] = value
        return return_dict

    @staticmethod
    def js_for_dict(object_to_parse: dict) -> dict:
        return_dict = {}
        for key, value in object_to_parse.items():
            if isinstance(value, str):
                value: str
                if value.lower() in ["true", "false"]:
                    return_dict[key] = True if value.lower() == "true" else False
                    continue

            return_dict[key] = value
        return return_dict



import os

from qtpy.QtCore import QEventLoop, Qt, QUrl
from qtpy.QtWebChannel import QWebChannel
from qtpy.QtWebEngineWidgets import QWebEngineView, QWebEnginePage


class MapWidget(QWebEngineView):
    """
    The MapWidget class is a QWebEngineView that houses the leaflet map.
    Since it is a QWidget, it can be added to any QLayout.
    """

    @property
    def page(self):
        return self._page

    @property
    def channel(self):
        return self._channel

    def __init__(self, use_file_absolute_path: bool = True, alternative_base_path: str=""):
        super().__init__()
        if use_file_absolute_path or len(alternative_base_path) == 0:
            self.base_path = os.path.dirname(os.path.abspath(__file__))
        else:
            self.base_path = alternative_base_path
        self._page = QWebEnginePage()
        self.setPage(self._page)
        self._channel = QWebChannel()
        self._page.setWebChannel(self._channel)
        self._loadPage()
        self.setContextMenuPolicy(Qt.NoContextMenu)

    def _get_page_path(self):
        return os.path.join(self.base_path, 'web', 'map.html')

    def _loadPage(self):
        html_path = self._get_page_path()
        # QEventLoop is used to make the page loading behave syncronously
        init_loop = QEventLoop()
        self._page.loadFinished.connect(init_loop.quit)
        self._page.load(QUrl().fromLocalFile(html_path))
        init_loop.exec_()

